@@ -126,7 +126,7 @@ void KateHighlighting::cleanup ()
   internalIDList.clear();
 }
 
-KateHlContext *KateHighlighting::generateContextStack (QVector<short> &contextStack,
+KateHlContext *KateHighlighting::generateContextStack (Kate::TextLineData::ContextStack &contextStack,
                                                        KateHlContextModification modification,
                                                        int &indexLastContextPreviousLine)
 {
@@ -250,33 +250,27 @@ void KateHighlighting::dropDynamicContexts()
   startctx = base_startctx;
 }
 
-/**
- * Parse the text and fill in the context array and folding list array
- *
- * @param prevLine The previous line, the context array is picked up from that if present.
- * @param textLine The text line to parse
- * @param foldingList will be filled
- * @param ctxChanged will be set to reflect if the context changed
- */
-void KateHighlighting::doHighlight ( Kate::TextLineData *prevLine,
+void KateHighlighting::doHighlight ( const Kate::TextLineData *prevLine,
                                      Kate::TextLineData *textLine,
-                                     QVector<int> &foldingList,
-                                     bool &ctxChanged )
+                                     const Kate::TextLineData *nextLine,
+                                     bool &ctxChanged,
+                                     int tabWidth )
 {
   if (!textLine)
     return;
 
   // in all cases, remove old hl, or we will grow to infinite ;)
   textLine->clearAttributes ();
+  
+  // reset folding start
+  textLine->clearMarkedAsFoldingStart ();
 
   // no hl set, nothing to do more than the above cleaning ;)
-  if (noHl) {
-    textLine->addAttribute (0, textLine->length(), KTextEditor::HighlightInterface::dsNormal);
+  if (noHl)
     return;
-  }
 
   // duplicate the ctx stack, only once !
-  QVector<short> ctx (prevLine->ctxArray());
+  Kate::TextLineData::ContextStack ctx (prevLine->contextStack());
 
   int previousLine = -1;
   KateHlContext *context;
@@ -335,6 +329,15 @@ void KateHighlighting::doHighlight ( Kate::TextLineData *prevLine,
     if (context->emptyLineContext)
         context = generateContextStack (ctx, context->emptyLineContextModification, previousLine);
   } else {
+    /**
+     * check if the folding begin/ends are balanced!
+     * constructed on demand!
+     */
+    QHash<short, int> *foldingStartToCount = 0;
+    
+    /**
+     * loop over line content!
+     */
     QChar lastDelimChar = 0;
     while (offset < len)
     {
@@ -383,50 +386,12 @@ void KateHighlighting::doHighlight ( Kate::TextLineData *prevLine,
 
         if (offset2 <= offset)
           continue;
+
         // BUG 144599: Ignore a context change that would push the same context
         // without eating anything... this would be an infinite loop!
         if ( item->lookAhead && ( item->ctx.pops < 2 && item->ctx.newContext == ( ctx.isEmpty() ? 0 : ctx.last() ) ) )
           continue;
 
-        if (item->region2)
-        {
-          // kDebug(13010)<<QString("Region mark 2 detected: %1").arg(item->region2);
-          if ( !foldingList.isEmpty() && ((item->region2 < 0) && (int)foldingList[foldingList.size()-2] == -item->region2 ) )
-          {
-            foldingList.resize (foldingList.size()-2);
-          }
-          else
-          {
-            foldingList.resize (foldingList.size()+2);
-            foldingList[foldingList.size()-2] = (uint)item->region2;
-            if (item->region2<0) //check not really needed yet
-              foldingList[foldingList.size()-1] = offset2;
-            else
-              foldingList[foldingList.size()-1] = offset;
-          }
-
-        }
-
-        if (item->region)
-        {
-          // kDebug(13010)<<QString("Region mark detected: %1").arg(item->region);
-
-        /* if ( !foldingList->isEmpty() && ((item->region < 0) && (*foldingList)[foldingList->size()-1] == -item->region ) )
-          {
-            foldingList->resize (foldingList->size()-1, QGArray::SpeedOptim);
-          }
-          else*/
-          {
-            foldingList.resize (foldingList.size()+2);
-            foldingList[foldingList.size()-2] = item->region;
-            if (item->region<0) //check not really needed yet
-              foldingList[foldingList.size()-1] = offset2;
-            else
-              foldingList[foldingList.size()-1] = offset;
-          }
-
-        }
-
         // regenerate context stack if needed
         context = generateContextStack (ctx, item->ctx, previousLine);
 
@@ -452,11 +417,42 @@ void KateHighlighting::doHighlight ( Kate::TextLineData *prevLine,
         {
           if (offset2 > len)
             offset2 = len;
-
-          // even set attributes ;)
+          
+          // handle folding end or begin
+          if (item->region || item->region2) {
+            /**
+             * for each end region, decrement counter for that type, erase if count reaches 0!
+             */
+            if (item->region2 && foldingStartToCount) {
+              QHash<short, int>::iterator end = foldingStartToCount->find (-item->region2);
+              if (end != foldingStartToCount->end()) {
+                if (end.value() > 1)
+                  --(end.value());
+                else
+                  foldingStartToCount->erase (end);
+              }
+            }
+            
+            /**
+             * increment counter for each begin region!
+             */
+            if (item->region) {
+              // construct on demand!
+              if (!foldingStartToCount)
+                foldingStartToCount = new QHash<short, int> ();
+              
+              ++(*foldingStartToCount)[item->region];
+            }
+          }
+          
+          // even set attributes or end of region! ;)
           int attribute = item->onlyConsume ? context->attr : item->attr;
-          if (attribute > 0)
-            textLine->addAttribute (offset, offset2-offset, attribute);
+          if (attribute > 0 || item->region2)
+            textLine->addAttribute (Kate::TextLineData::Attribute (offset, offset2-offset, attribute, item->region2));
+          
+          // create 0 length attribute for begin of region, if any!
+          if (item->region)
+            textLine->addAttribute (Kate::TextLineData::Attribute (offset2, 0, attribute, item->region));
 
           offset = offset2;
           lastChar = text[offset-1];
@@ -493,46 +489,77 @@ void KateHighlighting::doHighlight ( Kate::TextLineData *prevLine,
       {
         // set attribute if any
         if (context->attr > 0)
-          textLine->addAttribute (offset, 1, context->attr);
+          textLine->addAttribute (Kate::TextLineData::Attribute (offset, 1, context->attr, 0));
 
         lastChar = text[offset];
         offset++;
       }
     }
+    
+    /**
+     * check if folding is not balanced and we have more starts then ends
+     * then this line is a possible folding start!
+     */
+    if (foldingStartToCount) {
+      /**
+       * possible folding start, if imbalanced, aka hash not empty!
+       */
+      if (!foldingStartToCount->isEmpty())
+        textLine->markAsFoldingStartAttribute ();
+      
+      /**
+       * kill hash
+       */
+      delete foldingStartToCount;
+      foldingStartToCount = 0;
+    }
   }
   
-  // has the context stack changed ?
-  if (ctx == textLine->ctxArray())
-  {
-    ctxChanged = false;
-  }
-  else
-  {
-    ctxChanged = true;
-
-    // assign ctx stack !
-    textLine->setContext(ctx);
+  /**
+   * has the context stack changed?
+   */
+  if ((ctxChanged = (ctx != textLine->contextStack()))) {
+    /**
+     * try to share the simple stack that contains only 0
+     */
+    static const Kate::TextLineData::ContextStack onlyDefaulContext (1, 0);
+    if (ctx == onlyDefaulContext)
+      textLine->setContextStack(onlyDefaulContext);
+    
+    /**
+     * next try: try to share data with last line
+     */
+    else if (ctx == prevLine->contextStack())
+      textLine->setContextStack(prevLine->contextStack());
+    
+    /**
+     * ok, really use newly constructed stack!
+     */
+    else
+      textLine->setContextStack(ctx);
   }
 
   // write hl continue flag
   textLine->setHlLineContinue (item && item->lineContinue());
 
-  if (m_foldingIndentationSensitive) {
-    bool noindent=false;
-    for(int i=ctx.size()-1; i>=0; --i) {
+  // check for indentation based folding
+  if (m_foldingIndentationSensitive && (tabWidth > 0) && !textLine->markedAsFoldingStartAttribute ()) {
+    bool skipIndentationBasedFolding = false;
+    for(int i = ctx.size() - 1; i >= 0; --i) {
       if (contextNum(ctx[i])->noIndentationBasedFolding) {
-        noindent=true;
+        skipIndentationBasedFolding = true;
         break;
       }
     }
-    textLine->setNoIndentBasedFolding(noindent);
+    
+    /**
+     * compute if we increase indentation in next line
+     */
+    if (!skipIndentationBasedFolding && !isEmptyLine (textLine) && !isEmptyLine (nextLine)
+        && (textLine->indentDepth (tabWidth) < nextLine->indentDepth (tabWidth))) 
+       textLine->markAsFoldingStartIndentation ();
   }
-
-  //set the dsNormal attribute if we haven't found anything else
-  if(textLine->attributesList().empty()) {
-    textLine->addAttribute (0, textLine->length(), KTextEditor::HighlightInterface::dsNormal);
-  }
-
+  
   // invalidate caches
   for ( int i = 0; i < cachingItems.size(); ++i) {
     cachingItems[i]->cachingHandled = false;
@@ -1840,7 +1867,9 @@ int KateHighlighting::addToContextList(const QString &ident, int ctx0)
   readGlobalKeywordConfig();
   readWordWrapConfig();
 
-  readFoldingConfig ();
+  // only read for ourself
+  if (identifier == ident)
+    readFoldingConfig ();
 
   readSpellCheckingConfig();
 
@@ -2124,6 +2153,21 @@ QStringList KateHighlighting::getEmbeddedHighlightingModes() const
   return embeddedHighlightingModes;
 }
 
+bool KateHighlighting::isEmptyLine(const Kate::TextLineData *textline) const
+{
+  const QString &txt=textline->string();
+  if (txt.isEmpty())
+    return true;
+  
+  QLinkedList<QRegExp> l;
+  l=emptyLines(textline->attribute(0));
+  if (l.isEmpty()) return false;
+  foreach(const QRegExp &re,l) {
+    if (re.exactMatch(txt)) return true;
+  }
+  return false;
+}
+
 //END
 
 // kate: space-indent on; indent-width 2; replace-tabs on;
