@@ -284,8 +284,7 @@ KateHlKeyword::KateHlKeyword (int attribute, KateHlContextModification context,
 
 KateHlKeyword::~KateHlKeyword ()
 {
-  for (int i=0; i < dict.size(); ++i)
-    delete dict[i];
+  qDeleteAll(dict);
 }
 
 void KateHlKeyword::addList(const QStringList& list)
@@ -682,7 +681,7 @@ KateHlItem *KateHlRegExpr::clone(const QStringList *args)
   ret->dynamicChild = true;
   return ret;
 }
-//END
+// //END
 
 //BEGIN KateHlLineContinue
 KateHlLineContinue::KateHlLineContinue(int attribute, KateHlContextModification context, signed char regionId,signed char regionId2)
