@@ -1032,12 +1032,14 @@ KateViewConfig::KateViewConfig ()
    m_persistentSelectionSet (true),
    m_viInputModeSet (true),
    m_viInputModeStealKeysSet (true),
+   m_viInputModeEmulateCommandBarSet(false),
    m_automaticCompletionInvocationSet (true),
    m_wordCompletionSet (true),
    m_wordCompletionMinimalWordLengthSet (true),
    m_smartCopyCutSet (true),
    m_scrollPastEndSet (true),
    m_allowMarkMenu (true),
+   m_wordCompletionRemoveTailSet (false),
    m_view (0)
 {
   s_global = this;
@@ -1069,12 +1071,14 @@ KateViewConfig::KateViewConfig (KateView *view)
    m_persistentSelectionSet (false),
    m_viInputModeSet (false),
    m_viInputModeStealKeysSet (false),
+   m_viInputModeEmulateCommandBarSet(false),
    m_automaticCompletionInvocationSet (false),
    m_wordCompletionSet (false),
    m_wordCompletionMinimalWordLengthSet (false),
    m_smartCopyCutSet (false),
    m_scrollPastEndSet (false),
    m_allowMarkMenu (true),
+   m_wordCompletionRemoveTailSet (false),
    m_view (view)
 {
 }
@@ -1131,11 +1135,12 @@ void KateViewConfig::readConfig ( const KConfigGroup &config)
 
   setViInputMode (config.readEntry( "Vi Input Mode", false));
   setViInputModeStealKeys (config.readEntry( "Vi Input Mode Steal Keys", false));
-  setViInputModeHideStatusBar (config.readEntry( "Vi Input Mode Hide Status Bar", false));
+  setViInputModeEmulateCommandBar (config.readEntry( "Vi Input Mode Emulate Command Bar", false));
 
   setAutomaticCompletionInvocation (config.readEntry( "Auto Completion", true ));
   setWordCompletion (config.readEntry( "Word Completion", true ));
   setWordCompletionMinimalWordLength (config.readEntry( "Word Completion Minimal Word Length", 3 ));
+  setWordCompletionRemoveTail (config.readEntry( "Word Completion Remove Tail", true ));
   setSmartCopyCut (config.readEntry( "Smart Copy Cut", false ));
   setScrollPastEnd (config.readEntry( "Scroll Past End", false ));
 
@@ -1191,6 +1196,7 @@ void KateViewConfig::writeConfig (KConfigGroup &config)
   config.writeEntry( "Auto Completion", automaticCompletionInvocation());
   config.writeEntry( "Word Completion", wordCompletion());
   config.writeEntry( "Word Completion Minimal Word Length", wordCompletionMinimalWordLength());
+  config.writeEntry( "Word Completion Remove Tail", wordCompletionRemoveTail());
 
   config.writeEntry( "Smart Copy Cut", smartCopyCut() );
   config.writeEntry( "Scroll Past End" , scrollPastEnd() );
@@ -1199,7 +1205,7 @@ void KateViewConfig::writeConfig (KConfigGroup &config)
 
   config.writeEntry( "Vi Input Mode Steal Keys", viInputModeStealKeys());
 
-  config.writeEntry( "Vi Input Mode Hide Status Bar", viInputModeHideStatusBar());
+  config.writeEntry( "Vi Input Mode Emulate Command Bar", viInputModeEmulateCommandBar());
 
 
   if (isGlobal()) {
@@ -1565,15 +1571,6 @@ void KateViewConfig::setViInputMode (bool on)
   m_viInputModeSet = true;
   m_viInputMode = on;
 
-  // update all views and show/hide the status bar
-  foreach (KateView* view, KateGlobal::self()->views() ) {
-    if (on && !m_viInputModeHideStatusBar) {
-      view->showViModeBar();
-    } else {
-      view->hideViModeBar();
-    }
-  }
-
   // make sure to turn off edits mergin when leaving vi input mode
   if (!on && m_view) {
     m_view->doc()->setUndoMergeAllEdits(false);
@@ -1600,34 +1597,23 @@ void KateViewConfig::setViInputModeStealKeys (bool on)
   configEnd ();
 }
 
-bool KateViewConfig::viInputModeHideStatusBar () const
-{
-  if (m_viInputModeHideStatusBarSet || isGlobal())
-    return m_viInputModeHideStatusBar;
 
-  return s_global->viInputModeHideStatusBar();
+bool KateViewConfig::viInputModeEmulateCommandBar() const
+{
+  if (m_viInputModeEmulateCommandBarSet || isGlobal())
+    return m_viInputModeEmulateCommandBar;
+
+  return s_global->viInputModeEmulateCommandBar();
 }
 
-void KateViewConfig::setViInputModeHideStatusBar (bool on)
+void KateViewConfig::setViInputModeEmulateCommandBar(bool on)
 {
-  configStart ();
-
-  m_viInputModeHideStatusBarSet = true;
-  m_viInputModeHideStatusBar = on;
-
-  // update all views and show/hide the status bar
-  foreach (KateView* view, KateGlobal::self()->views() ) {
-    if (on && m_viInputMode) {
-      view->hideViModeBar();
-    } else if (viInputMode()) {
-      view->showViModeBar();
-    }
-  }
-
-  configEnd ();
+  configStart();
+  m_viInputModeEmulateCommandBarSet = true;
+  m_viInputModeEmulateCommandBar = on;
+  configEnd();
 }
 
-
 bool KateViewConfig::automaticCompletionInvocation () const
 {
   if (m_automaticCompletionInvocationSet || isGlobal())
@@ -1682,6 +1668,24 @@ void KateViewConfig::setWordCompletionMinimalWordLength (int length)
   configEnd ();
 }
 
+bool KateViewConfig::wordCompletionRemoveTail () const
+{
+  if (m_wordCompletionRemoveTailSet || isGlobal())
+  {
+    return m_wordCompletionRemoveTail;
+    }
+  return s_global->wordCompletionRemoveTail();
+}
+
+void KateViewConfig::setWordCompletionRemoveTail (bool on)
+{
+  configStart ();
+  m_wordCompletionRemoveTailSet = true;
+  m_wordCompletionRemoveTail = on;
+
+  configEnd ();
+}
+
 bool KateViewConfig::smartCopyCut () const
 {
   if (m_smartCopyCutSet || isGlobal())
