@@ -59,9 +59,10 @@
 #include "snippet/katesnippetglobal.h"
 #include "snippet/snippetcompletionmodel.h"
 #include "katemessagewidget.h"
-#include "messageinterface.h"
 #include "katetemplatehandler.h"
 
+#include <ktexteditor/messageinterface.h>
+
 #include <kparts/event.h>
 
 #include <kconfig.h>
