@@ -90,6 +90,7 @@ class KateViewInternal : public QWidget
     uint editSessionNumber;
     bool editIsRunning;
     KTextEditor::Cursor editOldCursor;
+    KTextEditor::Range editOldSelection;
   //END
 
   //BEGIN TAG & CLEAR & UPDATE STUFF
@@ -231,7 +232,6 @@ class KateViewInternal : public QWidget
   private Q_SLOTS:
     void slotRegionVisibilityChanged();
     void slotRegionBeginEndAddedRemoved(unsigned int);
-    void slotCodeFoldingChanged();
 
   private:
     void moveChar( Bias bias, bool sel );
@@ -294,8 +294,6 @@ class KateViewInternal : public QWidget
       QDrag*   dragObject;
     } m_dragInfo;
 
-    uint m_iconBorderHeight;
-
     //
     // line scrollbar + first visible (virtual) line in the current view
     //
