@@ -37,6 +37,7 @@
 #include "katevivisualmode.h"
 #include "katevireplacemode.h"
 #include "katevikeyparser.h"
+#include "kateviemulatedcommandbar.h"
 
 KateViInputModeManager::KateViInputModeManager(KateView* view, KateViewInternal* viewInternal)
 {
@@ -69,6 +70,10 @@ KateViInputModeManager::KateViInputModeManager(KateView* view, KateViewInternal*
            this,
            SLOT(markChanged(KTextEditor::Document*, KTextEditor::Mark,
                             KTextEditor::MarkInterface::MarkChangeAction)));
+  // We have to do this outside of KateViNormalMode, as we don't want
+  // KateViVisualMode (which inherits from KateViNormalMode) to respond
+  // to changes in the document as well.
+  m_viNormalMode->beginMonitoringDocumentChanges();
 }
 
 KateViInputModeManager::~KateViInputModeManager()
@@ -90,26 +95,33 @@ bool KateViInputModeManager::handleKeypress(const QKeyEvent *e)
     appendKeyEventToLog( copy );
   }
 
-  // FIXME: I think we're making things difficult for ourselves here.  Maybe some
-  //        more thought needs to go into the inheritance hierarchy.
-  switch(m_currentViMode) {
-  case NormalMode:
-    res = m_viNormalMode->handleKeypress(e);
-    break;
-  case InsertMode:
-    res = m_viInsertMode->handleKeypress(e);
-    break;
-  case VisualMode:
-  case VisualLineMode:
-  case VisualBlockMode:
-    res = m_viVisualMode->handleKeypress(e);
-    break;
-  case ReplaceMode:
-    res = m_viReplaceMode->handleKeypress(e);
-    break;
-  default:
-    kDebug( 13070 ) << "WARNING: Unhandled keypress";
-    res = false;
+  if (m_view->viModeEmulatedCommandBar()->isVisible())
+  {
+    res = m_view->viModeEmulatedCommandBar()->handleKeyPress(e);
+  }
+  else
+  {
+    // FIXME: I think we're making things difficult for ourselves here.  Maybe some
+    //        more thought needs to go into the inheritance hierarchy.
+    switch(m_currentViMode) {
+      case NormalMode:
+        res = m_viNormalMode->handleKeypress(e);
+        break;
+      case InsertMode:
+        res = m_viInsertMode->handleKeypress(e);
+        break;
+      case VisualMode:
+      case VisualLineMode:
+      case VisualBlockMode:
+        res = m_viVisualMode->handleKeypress(e);
+        break;
+      case ReplaceMode:
+        res = m_viReplaceMode->handleKeypress(e);
+        break;
+      default:
+        kDebug( 13070 ) << "WARNING: Unhandled keypress";
+        res = false;
+    }
   }
 
   return res;
@@ -246,12 +258,25 @@ void KateViInputModeManager::repeatLastChange()
 
 const QString KateViInputModeManager::getLastSearchPattern() const
 {
-  return m_view->searchPattern();
+  if (!KateViewConfig::global()->viInputModeEmulateCommandBar())
+  {
+    return m_view->searchPattern();
+  }
+  else
+  {
+    return m_lastSearchPattern;
+  }
 }
 
 void KateViInputModeManager::setLastSearchPattern( const QString &p )
 {
-  m_view->setSearchPattern(p);
+  if (!KateViewConfig::global()->viInputModeEmulateCommandBar())
+  {
+    // This actually triggers a search, so we definitely don't want it to be called
+    // if we are handle searches ourselves in the Emulated Command Bar.
+    m_view->setSearchPattern(p);
+  }
+  m_lastSearchPattern = p;
 }
 
 void KateViInputModeManager::changeViMode(ViMode newMode)
@@ -548,13 +573,16 @@ void KateViInputModeManager::addMark( KateDocument* doc, const QChar& mark, cons
   m_marks.insert( mark, doc->newMovingCursor( pos, behavior ) );
 
   // Showing what mark we set:
-  if ( showmark && mark != '>' && mark != '<' ) {
+  if ( showmark && mark != '>' && mark != '<' && mark != '[' && mark != '.' && mark != ']') {
     if( !marktype & KTextEditor::MarkInterface::markType01 ) {
       m_view->doc()->addMark( pos.line(),
           KTextEditor::MarkInterface::markType01 );
     }
 
-    m_viNormalMode->message("Mark set: " + mark);
+    // only show message for active view
+    if (m_view->doc()->activeView() == m_view) {
+      m_viNormalMode->message(i18n ("Mark set: %1", mark));
+    }
   }
 
   m_mark_set_inside_viinputmodemanager = false;
@@ -592,7 +620,7 @@ void KateViInputModeManager::markChanged (KTextEditor::Document* doc,
         }
       }
       if (!char_exist)
-        m_viNormalMode->error("There no more chars for the next bookmark");
+        m_viNormalMode->error(i18n ("There are no more chars for the next bookmark."));
     }
   }
 }
@@ -653,3 +681,31 @@ QString KateViInputModeManager::getMarksOnTheLine(int line) {
 
   return res;
 }
+
+
+QString KateViInputModeManager::modeToString(ViMode mode)
+{
+  QString modeStr;
+  switch (mode) {
+    case InsertMode:
+      modeStr = i18n("VI: INSERT MODE");
+      break;
+    case NormalMode:
+      modeStr = i18n("VI: NORMAL MODE");
+      break;
+    case VisualMode:
+      modeStr = i18n("VI: VISUAL");
+      break;
+    case VisualBlockMode:
+      modeStr = i18n("VI: VISUAL BLOCK");
+      break;
+    case VisualLineMode:
+      modeStr = i18n("VI: VISUAL LINE");
+      break;
+    case ReplaceMode:
+      modeStr = i18n("VI: REPLACE");
+      break;
+  }
+
+  return modeStr;
+}
